"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = useDeepCompareEffect;
var _react = _interopRequireDefault(require("react"));
var _isEqual = _interopRequireDefault(require("lodash/isEqual"));
function _interopRequireDefault(e) { return e && e.__esModule ? e : { default: e }; }
/**
 * Deep compare effect hook.
 * Ensures the effect runs on the first render and whenever dependencies deeply change.
 *
 * @param effect The effect callback function.
 * @param deps The dependencies array to compare deeply.
 */
function useDeepCompareEffect(effect, deps) {
  // Ref to track if it's the first render
  const firstRenderRef = _react.default.useRef(true);

  // Memoized dependencies to avoid redundant `isEqual` checks
  const memoizedDependencies = _react.default.useMemo(() => deps, [deps]);

  // Ref to store the previous dependencies
  const dependenciesRef = _react.default.useRef(memoizedDependencies);

  // Check for dependency changes
  const dependenciesChanged = !(0, _isEqual.default)(dependenciesRef.current, memoizedDependencies);
  if (dependenciesChanged) {
    dependenciesRef.current = memoizedDependencies;
  }
  _react.default.useEffect(() => {
    if (firstRenderRef.current) {
      firstRenderRef.current = false;
    }
    return effect();
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [dependenciesChanged]); // exclude the effect function from the dependencies
}
//# sourceMappingURL=useDeepCompareEffect.js.map